@@ -33,21 +33,52 @@ class DirectoryWalkerGuard(object):
         os.chdir(os.path.pardir)
 
 
+_default_pybind11_repr_re = re.compile(r'<\w+(\.\w+)* object at 0x[0-9a-f]+>')
+
+
+def replace_default_pybind11_repr_with_ellipses(line):
+    default_reprs = []
+
+    def replacement(m):
+        default_reprs.append(m.group(0))
+        return "..."
+
+    return default_reprs, _default_pybind11_repr_re.sub(replacement, line)
+
+
 class FunctionSignature(object):
-    # When True keeps generation of stubs with invalid signatures found in docstrings
-    # (yes, global variables, blame me)
-    non_stop_mode = False
+    # When True don't raise an error when invalid signatures/defaultargs are
+    # encountered (yes, global variables, blame me)
+    ignore_invalid_signature = False
+    ignore_invalid_defaultarg = False
+
     signature_downgrade = True
 
+    # Number of invalid default values found so far
+    n_invalid_default_values = 0
+
     # Number of invalid signatures found so far
     n_invalid_signatures = 0
 
+    @classmethod
+    def n_fatal_errors(cls):
+        return ((0 if cls.ignore_invalid_defaultarg else cls.n_invalid_default_values) +
+                (0 if cls.ignore_invalid_signature else cls.n_invalid_signatures))
+
     def __init__(self, name, args='*args, **kwargs', rtype='None', validate=True):
         self.name = name
         self.args = args
         self.rtype = rtype
 
         if validate:
+            invalid_defaults, self.args = replace_default_pybind11_repr_with_ellipses(self.args)
+            if invalid_defaults:
+                FunctionSignature.n_invalid_default_values += 1
+                lvl = logging.WARNING if FunctionSignature.ignore_invalid_defaultarg else logging.ERROR
+                logger.log(lvl, "Default argument value(s) replaced with ellipses (...):")
+                for invalid_default in invalid_defaults:
+                    logger.log(lvl, "    {}".format(invalid_default))
+
             function_def_str = "def {sig.name}({sig.args}) -> {sig.rtype}: ...".format(sig=self)
             try:
                 ast.parse(function_def_str)
@@ -56,9 +87,9 @@ class FunctionSignature(object):
                 if FunctionSignature.signature_downgrade:
                     self.name = name
                     self.args = "*args, **kwargs"
-                    self.rtype = "Any"
-                    logger.error("Generated stubs signature is degraded to `(*args, **kwargs) -> Any` for")
-                    lvl = logging.ERROR
+                    self.rtype = "typing.Any"
+                    lvl = logging.WARNING if FunctionSignature.ignore_invalid_signature else logging.ERROR
+                    logger.log(lvl, "Generated stubs signature is degraded to `(*args, **kwargs) -> typing.Any` for")
                 else:
                     lvl = logging.WARNING
                     logger.warning("Ignoring invalid signature:")
@@ -123,7 +154,13 @@ class PropertySignature(object):
         return FunctionSignature.argument_type(FunctionSignature('name', self.setter_args).split_arguments()[1])
 
 
+# If true numpy.ndarray[int32[3,3]] will be reduced to numpy.ndarray
+BARE_NUPMY_NDARRAY = False
+
+
 def replace_numpy_array(match_obj):
+    if BARE_NUPMY_NDARRAY:
+        return "numpy.ndarray"
     numpy_type = match_obj.group("type")
     # pybind always append size of data type
     if numpy_type in ['int8', 'int16', 'int32', 'int64',
@@ -150,7 +187,8 @@ class StubsGenerator(object):
     INDENT = " " * 4
 
     GLOBAL_CLASSNAME_REPLACEMENTS = {
-        re.compile(r"numpy.ndarray\[(?P<type>[^\[\]]+)(\[(?P<shape>[^\[\]]+)\])?\]"): replace_numpy_array,
+        re.compile(
+            r"numpy.ndarray\[(?P<type>[^\[\]]+)(\[(?P<shape>[^\[\]]+)\])?(?P<extra>[^][]*)\]"): replace_numpy_array,
         re.compile(
             r"(?<!\w)(?P<type>Callable|Dict|[Ii]terator|[Ii]terable|List|Optional|Set|Tuple|Union')(?!\w)"): replace_typing_types
     }
@@ -331,6 +369,16 @@ class AttributeStubsGenerator(StubsGenerator):
                 )
             ]
 
+        # special case for modules
+        # https://github.com/sizmailov/pybind11-stubgen/issues/43
+        if type(self.attr) is type(os) and hasattr(self.attr, "__name__"):
+            return [
+                "{name} = {repr}".format(
+                    name=self.name,
+                    repr=self.attr.__name__
+                )
+            ]
+
         value_lines = repr(self.attr).split("\n")
         if len(value_lines) == 1:
             value = value_lines[0]
@@ -358,7 +406,9 @@ class AttributeStubsGenerator(StubsGenerator):
                    + ['"""']
 
     def get_involved_modules_names(self):  # type: () -> Set[str]
-        return set((self.attr.__class__.__module__,))
+        if type(self.attr) is type(os):
+            return {self.attr.__name__}
+        return {self.attr.__class__.__module__}
 
 
 class FreeFunctionStubsGenerator(StubsGenerator):
@@ -550,6 +600,9 @@ class ClassStubsGenerator(StubsGenerator):
         for f in self.methods:  # type: ClassMemberStubsGenerator
             self.involved_modules_names |= f.get_involved_modules_names()
 
+        for attr in self.fields:
+            self.involved_modules_names |= attr.get_involved_modules_names()
+
     def to_lines(self):  # type: () -> List[str]
 
         def strip_current_module_name(obj, module_name):
@@ -711,37 +764,18 @@ class ModuleStubsGenerator(StubsGenerator):
             # result.extend(map(self.indent, map(lambda m: "import {}".format(m), used_modules)))
             result.extend(map(lambda mod: "import {}".format(mod), used_modules))
 
-        if "numpy" in used_modules:
+        if "numpy" in used_modules and not BARE_NUPMY_NDARRAY:
             result += [
                 "_Shape = typing.Tuple[int, ...]"
             ]
 
-        # define __all__
+        # add space between imports and rest of module
+        result += [""]
 
-        _all_ = []
-        for name in self.imported_classes.keys():
-            _all_.append(name)
-
-        for c in self.classes:
-            _all_.append(c.klass.__name__)
-
-        for f in self.free_functions:
-            _all_.append(f.name)
-
-        for m in self.submodules:
-            _all_.append(m.short_name)
-
-        for a in self.attributes:
-            _all_.append(a.name)
-
-        all_is_defined = False
-
-        for attr in self.attributes:
-            if attr.name == "__all__":
-                all_is_defined = True
-
-        if not all_is_defined:
-            result.append("__all__  = [\n" + ",\n".join(map(lambda s: '"%s"' % s, _all_)) + "\n]")
+        globals_ = {}
+        exec("from {} import *".format(self.module.__name__), globals_)
+        all_ = set(globals_.keys()) - {"__builtins__"}
+        result.append("__all__ = [\n    " + ",\n    ".join(map(lambda s: '"%s"' % s, sorted(all_))) + "\n]\n\n")
 
         for x in itertools.chain(self.classes,
                                  self.free_functions,
@@ -807,21 +841,38 @@ def main():
     parser = ArgumentParser(prog='pybind11-stubgen', description="Generates stubs for specified modules")
     parser.add_argument("-o", "--output-dir", help="the root directory for output stubs", default="./stubs")
     parser.add_argument("--root-module-suffix", type=str, default="-stubs", dest='root_module_suffix',
-                        help="optional suffix to disambiguate from the "
-                             "original package")
+                        help="optional suffix to disambiguate from the original package")
     parser.add_argument("--root_module_suffix", type=str, default=None, dest='root_module_suffix_deprecated',
                         help="Deprecated.  Use `--root-module-suffix`")
     parser.add_argument("--no-setup-py", action='store_true')
-    parser.add_argument("--non-stop", action='store_true', help="Don't stop when encountered invalid signatures")
+    parser.add_argument("--non-stop", action='store_true', help="Deprecated. Use `--ignore-invalid=all`")
+    parser.add_argument("--ignore-invalid", nargs="+", choices=["signature", "defaultarg", "all"], default=[],
+                        help="Ignore invalid specified python expressions in docstrings")
     parser.add_argument("--skip-signature-downgrade", action='store_true',
                         help="Do not downgrade invalid function signatures to func(*args, **kwargs)")
+    parser.add_argument("--bare-numpy-ndarray", action='store_true', default=False,
+                        help="Render `numpy.ndarray` without (non-standardized) bracket-enclosed type and shape info")
     parser.add_argument("module_names", nargs="+", metavar="MODULE_NAME", type=str, help="modules names")
     parser.add_argument("--log-level", default="INFO", help="Set output log level")
 
     sys_args = parser.parse_args()
 
     if sys_args.non_stop:
-        FunctionSignature.non_stop_mode = True
+        sys_args.ignore_invalid = ['all']
+        warnings.warn("`--non-stop` is deprecated in favor of `--ignore-invalid=all`", FutureWarning)
+
+    if sys_args.bare_numpy_ndarray:
+        global BARE_NUPMY_NDARRAY
+        BARE_NUPMY_NDARRAY = True
+
+    if 'all' in sys_args.ignore_invalid:
+        FunctionSignature.ignore_invalid_signature = True
+        FunctionSignature.ignore_invalid_defaultarg = True
+    else:
+        if 'signature' in sys_args.ignore_invalid:
+            FunctionSignature.ignore_invalid_signature = True
+        if 'defaultarg' in sys_args.ignore_invalid:
+            FunctionSignature.ignore_invalid_defaultarg = True
 
     if sys_args.skip_signature_downgrade:
         FunctionSignature.signature_downgrade = False
@@ -848,15 +899,22 @@ def main():
         for _module_name in sys_args.module_names:
             _module = ModuleStubsGenerator(_module_name)
             _module.parse()
-            if FunctionSignature.non_stop_mode or FunctionSignature.n_invalid_signatures == 0:
+            if FunctionSignature.n_fatal_errors() == 0:
                 _module.stub_suffix = sys_args.root_module_suffix
                 _module.write_setup_py = not sys_args.no_setup_py
                 recursive_mkdir_walker(_module_name.split(".")[:-1], lambda: _module.write())
 
         if FunctionSignature.n_invalid_signatures > 0:
-            logger.info("Useful link: Avoiding C++ types in docstrings: ")
+            logger.info("Useful link: Avoiding C++ types in docstrings:")
             logger.info("      https://pybind11.readthedocs.io/en/master/advanced/misc.html"
                         "#avoiding-cpp-types-in-docstrings")
+
+        if FunctionSignature.n_invalid_default_values > 0:
+            logger.info("Useful link: Default argument representation:")
+            logger.info("      https://pybind11.readthedocs.io/en/master/advanced/functions.html"
+                        "#default-arguments-revisited")
+
+        if FunctionSignature.n_fatal_errors() > 0:
             exit(1)
 
 
